use super::init::DB;
use serde::{Deserialize, Serialize};
use std::collections::HashMap;
use std::fmt;
use surrealdb::opt::Resource;

const CONCOUR: &str = "concour";

#[derive(Debug, Serialize, Deserialize, Clone)]
pub enum ConcourStatus {
    Created,
    Paused,
    OnGoing,
    Finished,
}

impl fmt::Display for ConcourStatus {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{:?}", self)
    }
}

#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct Concour {
    pub server_id: u64,
    pub enable: bool,
    pub channel_id: u64,
    pub title: String,
    pub description: String,
    pub start_date: chrono::DateTime<chrono::Utc>,
    pub periode: time::Duration,
    pub role_récompense: u64,
    pub keywords: Vec<String>,
    pub banner: Option<String>,
    pub index_keyword: u64,
    pub status: ConcourStatus,
    pub winner: HashMap<String, (u64, chrono::DateTime<chrono::Utc>)>,
}

impl Concour {
    pub fn new(server_id: u64, channel_id: u64, enable: bool) -> Result<Self, surrealdb::Error> {
        Ok(Self {
            server_id,
            enable,
            channel_id,
            title: String::new(),
            description: String::new(),
            start_date: chrono::Utc::now(),
            periode: time::Duration::days(0),
            role_récompense: 0,
            keywords: Vec::new(),
            banner: None,
            index_keyword: 0,
            status: ConcourStatus::Created,
            winner: HashMap::new(),
        })
    }
    pub async fn create(&self) -> Result<(), surrealdb::Error> {
        match DB.create(Resource::from(CONCOUR)).content(&self).await {
            Ok(_) => {}
            Err(e) => {
                return Err(e);
            }
        };
        Ok(())
    }
    pub async fn update(&self) -> Result<(), surrealdb::Error> {
        let sql = format!(
            "UPDATE {} SET enable = {}, title = '{}', description = '{}', start_date = '{}', periode = '{}', role_récompense = {}, keywords = '{:?}', banner = '{:?}', index_keyword = {}, status = '{}', winner = '{:?}' WHERE server_id = {} and channel_id = {}",
            CONCOUR, self.enable, self.title, self.description, self.start_date, self.periode, self.role_récompense, self.keywords, self.banner, self.index_keyword, self.status, self.winner, self.server_id, self.channel_id
        );
        match DB.query(sql).await {
            Ok(res) => {
                println!("{:?}", res);
            }
            Err(e) => {
                return Err(e);
            }
        };
        Ok(())
    }
    pub async fn find_by_server_id_channel_id(
        server_id: &u64,
        channel_id: &u64,
    ) -> Result<Option<Concour>, surrealdb::Error> {
        let sql = format!(
            "SELECT * FROM {} WHERE server_id = {} AND channel_id = {}",
            CONCOUR, server_id, channel_id
        );
        let mut results = match DB.query(&sql).await {
            Ok(results) => results,
            Err(e) => {
                return Err(e);
            }
        };
        let concour: Concour = match results.take(0) {
            Ok(Some(concour)) => concour,
            Ok(None) => {
                return Ok(None);
            }
            Err(e) => {
                return Err(e);
            }
        };
        Ok(Some(concour))
    }
    pub async fn find_by_server_id(server_id: &u64) -> Result<Vec<Concour>, surrealdb::Error> {
        let sql = format!("SELECT *  FROM {} WHERE server_id = {}", CONCOUR, server_id);
        let mut results = match DB.query(&sql).await {
            Ok(results) => results,
            Err(e) => {
                return Err(e);
            }
        };
        let mut concours = Vec::new();
        while let Ok(Some(concour)) = results.take(0) {
            concours.push(concour);
        }
        Ok(concours)
    }
}
